/*
 One can compile it with:
 mex gradBenvp.cpp LambertW/LambertW.cc
*/


#include <time.h>
#include <math.h>
#include <iostream>
#include <stdio.h>
#include "mex.h"
//#include <process.h> // only for windows
#include <pthread.h> // for linux

#define maxaj(a, b) (((a) > (b)) ? (a) : (b))
#define minaj(a, b) (((a) < (b)) ? (a) : (b))

typedef struct ST{
    double *gghx; // output
    double *xs; // estimate
	double *ys; // measurments
	double var; // ok
	int dif;
	
}thestruct; 

void* gradBenvpix( void* parm)
{

    thestruct* s = (thestruct *)parm;
	 
	double *gghx = (s)->gghx;
	double *xs = (s)->xs;
	double *ys = (s)->ys;
	int dif = (s)->dif;
	double var = (s)->var;


	for (int i = 0; i <dif; i++)
	{
        double T = 1 + 2*(xs[i]-ys[i]);
        double S = 2*(xs[i] + var);
        gghx[i] = (1- exp(-T/S));
	}
	
}


/*------------------------------------*/
void gradBenvp     (double* ghx,
                 double*  x,
                 double*  y,
                 double var,
                 double alpha,
                 double mean,   
                 int size 
				 )
{
   
	int NO_PROC = maxaj(1,3*sysconf( _SC_NPROCESSORS_ONLN ) / 4);
   
	pthread_t             *thread; // for linux
	
	thread = (pthread_t *) malloc(NO_PROC * sizeof(*thread));
	
	int j = 0;
	int dif = (int)(size/NO_PROC);
		
    for (int i = 0; i < NO_PROC ; ++i)
    {
	    void *status = 0;

		thestruct *st = NULL;
	    st = (thestruct *) malloc(sizeof(thestruct));
		
		st->xs = x + i*dif;
        st->ys = y+i*dif;
		st->gghx = ghx + i*dif;
		st->var = var;
		st->dif = dif;
		if (i == NO_PROC -1)
			st->dif = size - i*dif;

		pthread_create(&thread[i], NULL, gradBenvpix, (void *) st);
    }
	for (int i = 0; i < NO_PROC ; ++i)
	{
		//pthread_join(thread[NO_PROC-1], NULL);
		pthread_join(thread[i], NULL);
	}
	
     
}


/* the gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  double *y,*x, *exp_table_y, *exp_table_n,*ghx;
  double var, mean, alpha;
  
  size_t size, nmax;
  //printf ("1");
  
  /*  check for proper number of arguments */
  /* NOTE: You do not need an else statement when using mexErrMsgIdAndTxt
     within an if statement, because it will never get to the else
     statement if mexErrMsgIdAndTxt is executed. (mexErrMsgIdAndTxt breaks you out of
     the MEX-file) */

  if(nrhs!=8) 
    mexErrMsgIdAndTxt( "MATLAB:xtimesy:invalidNumInputs",
            "Five inputs required: x - vector (orginal signal); y - vector of observations; exp_table_y - vector defined as: exp_table_y[i] = exp(alpha/var (y[i] - mean) ); exp_table_n - vector defined as: exp_table_n[i] = exp(-n alpha^2/var); var - scalar (Gaussian noise variance; mean - scalar (Gaussian noise mean); alpha- scalar (scaling parameter of Poisson noise)"
			);
   //printf ("2");			
  if(nlhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:xtimesy:invalidNumOutputs",
            "One output required.");
  
  /* check to make sure the thd input argument is a scalar */
 //printf ("3");
  if( !mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]) ||
      mxGetN(prhs[4])*mxGetM(prhs[4])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:gradPG:varNotScalar",
            "Input var must be a scalar.");
  }
 
  /* check to make sure the mean input argument is a scalar */
   //printf ("4");
  if( !mxIsDouble(prhs[5]) || mxIsComplex(prhs[5]) ||
      mxGetN(prhs[5])*mxGetM(prhs[5])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:gradPG:meanNotScalar",
            "Input mean must be a scalar.");
  }
  //printf ("5");
  /* check to make sure the alpha input argument is a scalar */
  if( !mxIsDouble(prhs[6]) || mxIsComplex(prhs[6]) ||
      mxGetN(prhs[6])*mxGetM(prhs[6])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:gradPG:alphaNotScalar",
            "Input alpha must be a scalar.");
  }
   //printf ("6");
  /*  get the scalar input alpha */
  alpha = mxGetScalar(prhs[6]);
  
  /*  get the scalar input mean */
  mean = mxGetScalar(prhs[5]);
  
  /*  get the scalar input var */
  var = mxGetScalar(prhs[4]);
  
  /*  create a pointer to the input matrix y */
  exp_table_n = mxGetPr(prhs[3]);
  
  /*  create a pointer to the input matrix y */
  exp_table_y = mxGetPr(prhs[2]);
  
  /*  create a pointer to the input matrix y */
  y = mxGetPr(prhs[1]);
  
   /*  create a pointer to the input matrix x */
  x = mxGetPr(prhs[0]);
  // printf ("7");
  /*  get the dimensions of the matrix input y */
  size = maxaj(mxGetM(prhs[0]),mxGetN(prhs[0]));
  // printf ("8");
  
  
  nmax = maxaj(mxGetM(prhs[3]),mxGetN(prhs[3]));

 
  
  /*  set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix( size, 1, mxREAL);
  
  /*  create a C pointer to a copy of the output matrix */
  ghx = mxGetPr(plhs[0]);
  
 
  /*  call the C subroutine */

  //int t1 = clock();
  gradBenvp(ghx, x,y,var,alpha,mean,size);

				 
  //sleep(100);
  //int t2 = clock();
  //printf ("It took me %f seconds [%d, %d] .\n",((float)(t2-t1))/CLOCKS_PER_SEC, t1, t2);

}
