/*
 One can compile it with:
 mex prox_NLTVp.cpp LambertW/LambertW.cc
*/

#include <unistd.h>
#include <time.h>
#include "mex.h"
#include <pthread.h> // for linux
#include <math.h>


#define maxaj(a, b) (((a) > (b)) ? (a) : (b))
#define minaj(a, b) (((a) < (b)) ? (a) : (b))
#define SQ(x)	((x) * (x))

typedef struct ST{
    double *prox;
    double *y;
	double *degree;
	double gamma;
	int dif;
}thestruct; 

void* proxL2pix( void* parm)
{
    thestruct* s = (thestruct *)parm;
	 
	double *prox = (s)->prox;
	double *y = (s)->y;
	double *degree = (s)->degree;
	int dif = (s)->dif;
	double gamma = (s)->gamma;
	int totals = 0;
	for (int i = 0; i <dif; i++)
	{
	    int n = (int)(degree[i]);
		
		// L2 norm
		double norm = 0;
	
		for (int j = 0; j<n; j++)
		{
		  double t = y[j + totals];
		 
		  norm+= SQ(t);
		
		}
		
		norm = sqrt(norm);
		// treshold
		double tres = 0.0;
		if (norm!=0.0)
		{
			tres = 1 - gamma/norm;
		}
		tres = maxaj(tres,0.0);
		
		for (int j = 0; j<n; j++)
		{
			prox[j + totals] =  tres * y[j + totals];
			
		}
		totals+=n;
		
	}	

}


/*------------------------------------*/
void proxNLTV   (double* prox, // output vector of size = size_d
                 double*  y, // inout vector
                 double*  degree, // input vector
				 double gamma, // input gamma (scalar)
				 int size_d, // input size degree (total number of neighbours)
				 int size_y // input size of image y
                 )
{
   
	//int NO_PROC =  1; //maxaj(1,sysconf( _SC_NPROCESSORS_ONLN ) / 2);
	
	int NO_PROC =  maxaj(1,sysconf( _SC_NPROCESSORS_ONLN ) / 2);
 
	pthread_t             *thread; // for linux
	thread = (pthread_t *) malloc(NO_PROC * sizeof(*thread));
	int dif = (int)(floor(size_d/NO_PROC));
  
	
    for (int i = 0; i < NO_PROC ; ++i)
    {
	    void *status = 0;
		
		int totaln = 0;
		
		for (int j =0; j <i*dif; j++)
		{
			totaln+= degree[j];
		}
        
		thestruct *st = NULL;
	    st = (thestruct *) malloc(sizeof(thestruct));
		
		st-> prox = prox + totaln; //(prox + totaln[i]);
		st->degree = (degree + i*dif);
		st-> y = y + totaln; //(y + totaln[i]);
		st->dif = dif;
		if (i == (NO_PROC -1))
		{
		   st->dif = (size_d - i*dif);
		   //printf("last proc %d \n", i );
		}
		st->gamma = gamma;
		
	
				
        pthread_create(&thread[i], NULL, proxL2pix, (void *) st);
    }
	for (int i = 0; i < NO_PROC ; ++i)
	{
		pthread_join(thread[i], NULL);
	}
	

}
				 




				 

/* the gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  double *y,*degree,*prox;
  double gamma;  
  size_t size_d, size_y;

  
  /*  check for proper number of arguments */
  /* NOTE: You do not need an else statement when using mexErrMsgIdAndTxt
     within an if statement, because it will never get to the else
     statement if mexErrMsgIdAndTxt is executed. (mexErrMsgIdAndTxt breaks you out of
     the MEX-file) */
  if(nrhs!=5) 
    mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:invalidNumInputs",
            "Five inputs required: y - image(vector); degree - vector; gamma - scalar ; size(degree)<= size(y)); (nx, ny)-not used"
			);
  if(nlhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:invalidNumOutputs",
            "One output required.");
  
  /* check to make sure the thd input argument is a scalar */
  if( !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) ||
      mxGetN(prhs[2])*mxGetM(prhs[2])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:xNotScalar",
            "Input gamma must be a scalar.");
  }
  
  
  /*  get the scalar input alpha */
  gamma = mxGetScalar(prhs[2]);
   
  /*  create a pointer to the input matrix degree */
  degree = mxGetPr(prhs[1]);
  
   /*  create a pointer to the input matrix y */
  y = mxGetPr(prhs[0]);
  
  /*  get the dimensions of the matrix input y */
  size_y = maxaj(mxGetM(prhs[0]),mxGetN(prhs[0]));
  size_d = maxaj(mxGetM(prhs[1]),mxGetN(prhs[1]));
  
  if ((minaj(mxGetM(prhs[0]),mxGetN(prhs[0])) != 1) 
	||  (minaj(mxGetM(prhs[1]),mxGetN(prhs[1])) != 1) )
  {
      mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:xNotVector",
            "Input y, degree must be vectors.");
  }
  
  if ( size_d > size_y)
  {
      mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:incorrectSize",
            "Size of image y must be greater or equal than size of vector degree");
  }
  
 
  
  /*  set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix( size_y,1, mxREAL);
  
  /*  create a C pointer to a copy of the output matrix */
  prox = mxGetPr(plhs[0]);
  
  
  int sumdegree = 0;

  for(int i = 0 ; i < size_d ; ++i) { 
    sumdegree += (int)(degree[i]);
  }

  //printf("sumdegree=%d, size_y=%d\n", sumdegree, size_y);

  if ( sumdegree != size_y)
  {
      mexErrMsgIdAndTxt( "MATLAB:prox_NLTV:invalidNumInputs", "The sum of degrees is too large");
  }
  
  /*  call the C subroutine */

  proxNLTV( prox, y,degree,gamma, size_d,size_y);
 
}
