#include "logpoissgauss.h"
#include "mex.h"

#define maxaj(a, b) (((a) > (b)) ? (a) : (b))
#define minaj(a, b) (((a) < (b)) ? (a) : (b))

void inittabn(double* exp_table_n, double var, double alpha, int maxn)
{
    double coef = SQ(alpha)/var;
    //printf("max n [%d]", maxn );
    for(int i=1; i<maxn; ++i)
    {
        exp_table_n[i] = exp(- (i*coef));
        
    }
}


/* the gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  double *y,*x,*result;
  double var, mean, alpha;
  
  size_t size;
  
  /*  check for proper number of arguments */
  /* NOTE: You do not need an else statement when using mexErrMsgIdAndTxt
     within an if statement, because it will never get to the else
     statement if mexErrMsgIdAndTxt is executed. (mexErrMsgIdAndTxt breaks you out of
     the MEX-file) */
  if(nrhs!=4) 
    mexErrMsgIdAndTxt( "MATLAB:xtimesy:invalidNumInputs",
            "Five inputs required.");
  if(nlhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:xtimesy:invalidNumOutputs",
            "One output required.");
  
  /* check to make sure the thd input (var) argument is a scalar */
  if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
      mxGetN(prhs[1])*mxGetM(prhs[1])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:logpoissgauss:varNotScalar",
            "Input var must be a scalar.");
  }
  
  /* check to make sure the mean input argument is a scalar */
  if( !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) ||
      mxGetN(prhs[2])*mxGetM(prhs[2])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:logpoissgauss:alphaNotScalar",
            "Input mean must be a scalar.");
  }
  
  /* check to make sure the alpha input argument is a scalar */
  if( !mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[3])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:logpoissgauss:meanNotScalar",
            "Input alpha must be a scalar.");
  }
  
  
  
  /*  get the scalar input alpha */
  alpha = mxGetScalar(prhs[3]);
  
   /*  get the scalar input alpha */
  mean = mxGetScalar(prhs[2]);
   
  /*  get the scalar input var */
  var = mxGetScalar(prhs[1]);
  
  /*  create a pointer to the input matrix y */
  y = mxGetPr(prhs[0]);
  
  /*  get the dimensions of the matrix input y */
  size = maxaj(mxGetM(prhs[0]),mxGetN(prhs[0]));
  if ((minaj(mxGetM(prhs[0]),mxGetN(prhs[0])) != 1) )
  {
      mexErrMsgIdAndTxt( "MATLAB:xtimesy:xNotScalar",
            "Input y must be a vector.");
  }
 
  
  /*  set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix( 1, size, mxREAL);
  
  /*  create a C pointer to a copy of the output matrix */
  result = mxGetPr(plhs[0]);
  
  /* compute n_max */
  double ymax = 0;

  for (int i =0 ; i < size; ++i)
  {
      if (y[i]>ymax) ymax = y[i];
  }
  double alpha2var2 = SQ(alpha)/(var);
  double lam = exp(alpha*(ymax-mean)/var)*ymax*alpha2var2;
  double xstar =  LambertW(0, lam)/alpha2var2;
  double delta = 5*ceil(sqrt(var)/alpha)+1;
  int nmax = (int) ceil(xstar+delta);
  
  /*  call the C subroutine */
  init_exp_table_n(result, var,alpha,nmax);
}