#include "logpoissgauss.h"
#include "mex.h"

#define maxaj(a, b) (((a) > (b)) ? (a) : (b))
#define minaj(a, b) (((a) < (b)) ? (a) : (b))

// Initialize the lookup table
void inittaby(double *exp_table_y, double *y, int size, double var, double alpha, double mean)
{
	
    double coef = alpha/var;
  
    for(int i=0; i<size; ++i) {
       
        exp_table_y[i] = exp((y[i]-mean)*coef);
       
    }
}



/* the gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
   double *y,*x,*result;
  double var, mean, alpha;
  
  size_t size;
  
  /*  check for proper number of arguments */
  /* NOTE: You do not need an else statement when using mexErrMsgIdAndTxt
     within an if statement, because it will never get to the else
     statement if mexErrMsgIdAndTxt is executed. (mexErrMsgIdAndTxt breaks you out of
     the MEX-file) */
  if(nrhs!=4) 
    mexErrMsgIdAndTxt( "MATLAB:inittaby:invalidNumInputs",
            "Four inputs required: y - vector(observations); var - scalar; mean - scalar; alpha - scalar");
  if(nlhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:inittaby:invalidNumOutputs",
            "One output required.");
  
  /* check to make sure the thd input (var) argument is a scalar */
  if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
      mxGetN(prhs[1])*mxGetM(prhs[1])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:inittaby:xNotScalar",
            "Input var must be a scalar.");
  }
  
  /* check to make sure the mean input argument is a scalar */
  if( !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) ||
      mxGetN(prhs[2])*mxGetM(prhs[2])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:logpoissgauss:xNotScalar",
            "Input mean must be a scalar.");
  }
  
  /* check to make sure the alpha input argument is a scalar */
  if( !mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[3])!=1 ) {
    mexErrMsgIdAndTxt( "MATLAB:inittaby:xNotScalar",
            "Input alpha must be a scalar.");
  }
  
  
  
  /*  get the scalar input alpha */
  alpha = mxGetScalar(prhs[3]);
  
   /*  get the scalar input alpha */
  mean = mxGetScalar(prhs[2]);
   
  /*  get the scalar input var */
  var = mxGetScalar(prhs[1]);
  
  /*  create a pointer to the input matrix y */
  y = mxGetPr(prhs[0]);
  
  /*  get the dimensions of the matrix input y */
  size = maxaj(mxGetM(prhs[0]),mxGetN(prhs[0]));
  if ((minaj(mxGetM(prhs[0]),mxGetN(prhs[0])) != 1) )
  {
      mexErrMsgIdAndTxt( "MATLAB:xtimesy:xNotScalar",
            "Input y must be a vector.");
  }
  
  /*  get the dimensions of the matrix input y */
  size = maxaj(mxGetM(prhs[0]),mxGetN(prhs[0]));
  if ((minaj(mxGetM(prhs[0]),mxGetN(prhs[0])) != 1) )
  {
      mexErrMsgIdAndTxt( "MATLAB:xtimesy:xNotScalar",
            "Input y must be a vector.");
  }
 
  
  /*  set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix( 1, size, mxREAL);
  
  /*  create a C pointer to a copy of the output matrix */
  result = mxGetPr(plhs[0]);
    
  /*  call the C subroutine */
  inittaby(result, y, size, var, alpha, mean);
}


