%------------------------------------------------------- %
% Primal-Dual splitting algorithm [Combettes et al. 2011]%
%       Poisson-Gaussian data-fidelity term                       %
%------------------------------------------------------- %


function  [MAE, SNR,x] = primaldualrecPGNLTV(x0,y,yTrunc,h,NbIt,nx,ny, gvar, gmean, alpha,lambda,xmin,xmax,I, data_fidelity, prior, L, degree, gamma,tolerance)

% ----------------------------------------------------------------------- %
% f(x) = PG (Hx,y) + i_[xmin,xmax} + lambda * NLTV(x) 
% ----------------------------------------------------------------------- %

affiche = 500; %frequency display
x = x0; %Initialization primal variable
v1 = zeros(size(L,1),1); %Initialization dual variable
Lt = L'; % transposition of the linear operator L
disp(['lambda = ',num2str(lambda)]);   


switch data_fidelity
    case 'PoissGauss'
        tabn = inittabn(y,gvar,gmean,alpha,tolerance);
        taby = inittaby(y,gvar,gmean,alpha);
        funGrad = @gradPGp; %(Hx,y,taby,tabn,gvar,gmean,alpha,tolerance);
        algo = 'GradProx';
    case 'Gauss'
        tabn = 0;
        taby = 0;
        funGrad = @grad_gaussiandep; %(x,y,h,gvar,alpha, nx,ny)
        algo = 'GradProx';
    case 'GAST'
        tabn = 0;
        taby = 0;
        funGrad = @grad_gaussiangast; %(x,y,h, gvar,nx,ny); % grad GAST  
        algo = 'GradProx';
    case 'Poiss'
        algo = 'ProxProx';
        
        funProx = @prox_poisson; %(1/gamma, y22./gamma,y,v);   
    case 'SPoiss'    
        algo = 'ProxProx';
        
        funProx = @prox_poissonnization; %(1/gamma, y22./gamma,y,v);
    case 'EXP'
        tabn = 0;
        taby = 0;
        funGrad = @gradBenvp; %(Tx,y,taby,tabn, gvar,gmean, alpha)   
        algo = 'GradProx';
    case 'WL2'
        tabn = 0;
        taby = 0;
        funGrad = @grad_weightedl2; %(Tx,y,taby,tabn, gvar,gmean, alpha)   
        algo = 'GradProx';
end

switch prior
    case 'NLTV'
        funProxPrior = @prox_NLTVp; % (y21/gamma, degree, lambda/gamma); => (y21/gamma, degree, lambda/gamma,nx,ny);
    case 'TV'
        funProxPrior = @prox_tv;  %(1/gamma,y21/gamma,lambda,nx,ny); =>(y21/gamma, degree, lambda/gamma,nx,ny);
    case 'BM3D'
        funProxPrior = @proxL1; %(y21/gamma,lambda/gamma); ==> I should impelemnt it in C, parallel version is desiered! 
end

% ---------------------------------------------------------- %
% ----- ----- ----- ----- START ALGO ----- ----- ----- ----- %
% ---------------------------------------------------------- %
switch algo
    
    case 'ProxProx'
        
        v2 = zeros(nx*nx,1);  
        y = max(y,0.0001);
        
        for k = 1:NbIt
            
            Gsv1 = Lt*v1; % product of NLTV adjoint with dual variable v1
            Hv2 = ConvolutionAdj2D(v2,h,nx,ny);
            
            y1 = x - gamma * (Gsv1 + Hv2); % line 1
            p1 = max(min(y1,xmax),xmin); % line 2
            Gx = (x'*Lt)'; %m =1 (NLTV prox loop)   
            y21 = v1 + gamma *Gx;
            prox = funProxPrior (y21/gamma, degree, lambda/gamma,nx,ny);
            y21g = y21/gamma;
            p21 = y21 - gamma  * prox;
            Gp1 = (p1'*Lt)';
            q21 = p21 + gamma * Gp1;
            v1  = v1 - y21 + q21;
            
            Hx  = Convolution2D(x,h,nx,ny);  %m = 2 (Poisson prox loop)
            y22 = v2 + gamma * Hx;
            p22 = y22-gamma*funProx(1/gamma, y22./gamma,yTrunc,gvar);
            Hp1 = Convolution2D(p1,h,nx,ny); 
            q22 = p22 + gamma*Hp1;
            v2  = v2 - y22 + q22;    
            
            Gsp21 = (p21'*L)'; % q1 update
            Hp22  = ConvolutionAdj2D(p22,h,nx,ny);  
            q1 = p1 - gamma * (Gsp21 + Hp22);
            x = x - y1 + q1; % x update
            if(mod(k,affiche)==1) %display
                SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
                MAE = mean(abs(I(:)-x) );
                disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]);  
            end
        end
        % compute snr
        SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
        MAE = mean(abs(I(:)-x) );   
        
    case 'GradProx'
        
        for k = 1:NbIt
            
            Gsv1 = Lt*v1; %line 1
            Hx = Convolution2D(x,h, nx,ny);
            GradHx1 = funGrad(Hx,yTrunc,taby,tabn,gvar,gmean,alpha,tolerance);
            GradHx = ConvolutionAdj2D(GradHx1,h,nx,ny);
            %size(x)
            %size(Gsv1)
            %size(GradHx)
            y1 = x - gamma *(Gsv1 + GradHx);
            p1 = max(min(y1,xmax),xmin);      
            Gx = L*x; %m =1 (NLTV prox loop)
            y21 = v1 + gamma *Gx;
            prox = funProxPrior (y21/gamma, degree, lambda/gamma,nx,ny);
            y21g = y21/gamma;
            p21 = y21 - gamma  * prox;
            Gp1 = L*p1; %line 5 
            q21 = p21 + gamma * Gp1;
            v1  = v1 - y21 + q21;
            Gsp21 = Lt*p21; % q1 update
            Hp1 = 	Convolution2D(p1, h,nx,ny);
            GradHp11 = funGrad(Hp1,yTrunc,taby,tabn,gvar,gmean,alpha,tolerance); 
            GradHp1  = ConvolutionAdj2D(GradHp11,h,nx,ny);
            q1 = p1 - gamma * (Gsp21 + GradHp1);
            x = x - y1 + q1; % x update
            if(mod(k,affiche)==1)  % display 
                SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));
                MAE = mean(abs(I(:)-x) );
                disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]);    
                
            end
        end
        SNR = 10*log10(sum(I(:).^2)/sum((I(:)-x).^2));  % compute snr
        MAE = mean(abs(I(:)-x) );
end
disp(['k = ',num2str(k),' | SNR = ',num2str(SNR),' | MAE = ',num2str(MAE)]);   








 
